//Andnotor, developpez.net Defi N5 Sudoku solver
//------------------------------------------------------------------------------
 //Fiche des boutons de commandes (Voir aussi TfLayeredForm)
//------------------------------------------------------------------------------
 unit CommandButtons;

interface

uses
  Effects, LayeredImage,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, LayeredForm, Buttons,
  //SpeedButtonEx doit obligatoirement tre en dernier!
  SpeedButtonEx;

type
  TfCommandButtons = class(TfLayeredForm)
    bLoad: TSpeedButton;
    bNew: TSpeedButton;
    bSave: TSpeedButton;
    bIgnation: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure bNewClick(Sender: TObject);
    procedure bIgnationClick(Sender: TObject);
    procedure bSaveClick(Sender: TObject);
    procedure bLoadClick(Sender: TObject);

  private
    BkBmp       :TBitmap;       //Image de fond
    BtnBmp      :TBitmap;       //Image des boutons
    RedBtnBmp   :TBitmap;       //Bouton rouge
    OrangeBtnBmp:TBitmap;       //Bouton orange
    GreenBtnBmp :TBitmap;       //Bouton vert

  protected
    //Traitement des effets visuels et sonores.
    procedure DoEffect(const aEffect :TWMRendering); override;

  public
    procedure EnableButtons(aEnabled :boolean);
  end;

var
  fCommandButtons: TfCommandButtons;

//------------------------------------------------------------------------------

implementation

uses EffectsConst, Main, HAL, Display;

{$R *.dfm}

procedure TfCommandButtons.bIgnationClick(Sender: TObject);
begin
  inherited;
  EnableButtons(FALSE);
  fMain.Solve;
end;

procedure TfCommandButtons.bLoadClick(Sender: TObject);
begin
  inherited;
  fMain.Load;
end;

procedure TfCommandButtons.bNewClick(Sender: TObject);
begin
  inherited;
  fMain.New;
end;

procedure TfCommandButtons.bSaveClick(Sender: TObject);
begin
  inherited;
  fMain.Save;
end;

procedure TfCommandButtons.DoEffect(const aEffect: TWMRendering);
var
  Bmp    :TBitmap;
  R1, R2 :TRect;
  i      :integer;

begin
  inherited;

  if aEffect.State = esPlay then
  begin
    case aEffect.ID of
      PLAY_START     : PlayRendering(aEffect.ID, ControlCount, 0, 200);
      PLAY_STOP      : PlayRendering(aEffect.ID, ControlCount, 0, 200);
      DRAW_BUTTON    : PlayRendering(aEffect.ID, 1, aEffect.Param);
    end;

    Exit;
  end;

  case aEffect.ID of

    //-- Activation ------------------------------------------------------------
    
    PLAY_START1:        case aEffect.State of
                          esRenderingRun:   Controls[aEffect.Step -1].Enabled := TRUE;
                        end;

    //-- Dsactivation ---------------------------------------------------------

    PLAY_STOP1:         case aEffect.State of
                          esRenderingRun:   Controls[aEffect.Step -1].Enabled := FALSE;
                        end;

    //-- Dessin des boutons ----------------------------------------------------

    DRAW_BUTTONS:       case aEffect.State of
                          esRenderingRun:   for i := 0 to ControlCount -1 do
                                            begin
                                              R1 := Rect(0, 0, BtnBmp.Width, BtnBmp.Height);
                                              R2 := Controls[i].BoundsRect;

                                              Background.Draw(BtnBmp, R1, R2);
                                            end;

                          esRenderingEnd:   Background.Push;
                        end;

    //-- Dessin d'un bouton (ds)activ ----------------------------------------

    DRAW_BUTTON:        case aEffect.State of
                          esRenderingRun:   with TSpeedButton(aEffect.Param) do
                                            begin
                                              if Enabled then
                                              begin
                                                case NumGlyphs of
                                                  1 : Bmp := RedBtnBmp;
                                                  2 : Bmp := OrangeBtnBmp;
                                                  3 : Bmp := GreenBtnBmp;
                                                end;

                                                R1 := Rect(0, 0, BtnBmp.Width, BtnBmp.Height);
                                                R2 := Rect(0, 0, Bmp.Width, Bmp.Height);

                                                if Down
                                                then Background.Draw(Bmp, R1, BoundsRect, erDest, 255)
                                                else Background.Draw(Bmp, R1, BoundsRect, erDest, 150);
                                              end
                                              else Background.Erase(BoundsRect);
                                            end;
                        end;
  end;

  //Force la nouvelle image  l'cran
  Background.Flush;
end;

procedure TfCommandButtons.EnableButtons(aEnabled: boolean);
var
  i: Integer;
begin
  for i := 0 to ControlCount -1 do
    Controls[i].Enabled := aEnabled;
end;

procedure TfCommandButtons.FormCreate(Sender: TObject);
begin
  inherited;

  BkBmp        := LoadPngToBmp('COMMAND_BTN_BK');
  BtnBmp       := LoadPngToBmp('SQUARE_BTN');
  RedBtnBmp    := LoadPngToBmp('SQUARE_RED_BTN');
  OrangeBtnBmp := LoadPngToBmp('SQUARE_ORANGE_BTN');
  GreenBtnBmp  := LoadPngToBmp('SQUARE_GREEN_BTN');

  Background.SetBitmap(BkBmp);

  PlayRendering(DRAW_BUTTONS);
end;

procedure TfCommandButtons.FormDestroy(Sender: TObject);
begin
  BkBmp.Free;
  BtnBmp.Free;
  RedBtnBmp.Free;
  OrangeBtnBmp.Free;
  GreenBtnBmp.Free;

  inherited;
end;

end.
