//Andnotor, developpez.net Defi N5 Sudoku solver
//------------------------------------------------------------------------------
//Fiche de la porte (Voir aussi TfLayeredForm)
//------------------------------------------------------------------------------
unit Door;

interface

uses
  Effects, LayeredImage,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, LayeredForm;

type
  TfDoor = class(TfLayeredForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FIsOpen      :boolean;     //Porte ouverte ?

    BkBmp        :TBitmap;     //Image de fond
    DoorBmp      :TBitmap;     //Image de la porte
    LightBmp     :TBitmap;     //Lumire blanche
    RedLightBmp  :TBitmap;     //Lumire rouge

  protected
    //Traitement des effets visuels et sonores.
    procedure DoEffect(const aEffect :TWMRendering); override;

  public
    property IsOpen :boolean read FIsOpen;
  end;

var
  fDoor: TfDoor;

//------------------------------------------------------------------------------

implementation

uses EffectsConst, Main, HAL, Display;

{$R *.dfm}

procedure TfDoor.DoEffect(const aEffect :TWMRendering);
var
  //Permet d'inteprter aEffect comme un TWMSound sans transtypage
  Sound :TWMSound absolute aEffect;

  DoFlush :boolean;
  Offset  :integer;
  R1, R2  :TRect;
begin
  inherited;
  
  if aEffect.State = esPlay then
  begin
    case aEffect.ID of
      OPENDOOR :    //** Ouverture de la porte **
                         //- HAL parle
                         //- Lumire rouge
                         //  <synchro>
                         //- La porte s'enfonce
                         //  <synchro>
                         //- La porte coulisse
                         if not FIsOpen then
                           PlayRendering(aEffect.ID);

      CLOSEDOOR :   //** Fermeture de la porte **
                         //- HAL parle
                         //- La porte coulisse
                         //  <synchro>
                         //- La porte se referme
                         //  <synchro>
                         //- Lumire rouge
                         if FIsOpen then
                           PlayRendering(aEffect.ID);

//      PLAY_QUIT :        PlayRendering(aEffect.ID, 255, 0, aEffect.Param);
    end;

    Exit;
  end;

  DoFlush := TRUE;

  case aEffect.ID of
    INIT_DOOR:        case aEffect.State of
                        esRenderingRun: begin
                                          Background.Draw(DoorBmp);
                                          Background.Draw(bKBmp);
                                        end;
                      end;

    //-- Ouverture de la porte -------------------------------------------------

    OPENDOOR1:        //Dbut
                      case aEffect.State of
                        esRenderingStart: begin
                                            fDisplay.Write(sOpeningSudokuPad);
                                            Background.Push;
                                          end;

                        esRenderingRun:   begin
                                            Sleep(1000);
                                            PlayRendering(OPENDOOR2, 5);
                                          end;
                      end;

    OPENDOOR2:        //Allume la lumire rouge
                      case aEffect.State of
                        esRenderingRun:   begin
                                            PlaySound(0, 'LOCKDOOR', 0.2);
                                            Background.Draw(RedLightBmp, erAll, StepUp(aEffect));
                                          end;

                        esRenderingEnd:   PlayRendering(OPENDOOR3, 10);
                      end;

    OPENDOOR3:        //Eteind la lumire rouge
                      case aEffect.State of
                        esRenderingRun:   Background.Draw(RedLightBmp, erAll, StepDown(aEffect));

                        esRenderingEnd:   PlayRendering(OPENDOOR4, 40);
                      end;

    OPENDOOR4:        //La porte s'enfonce (la lumire blanche s'allume)
                      case aEffect.State of
                        esRenderingRun:   Background.Draw(LightBmp, erAll, StepUp(aEffect));

                        esRenderingEnd:   begin
                                            Background.Pop;
                                            Sleep(1000);
                                            PlayRendering(OPENDOOR5, 40);
                                          end;
                      end;

    OPENDOOR5:        //La porte coulisse (Descend) - Fin
                      case aEffect.State of
                        esRenderingStart: PlaySound(0, 'SLIDEDOOR');

                        esRenderingRun:   begin
                                            Offset := StepUp(aEffect, Height -40);

                                            R1 := Rect(0, 0, Width, Height -Offset);
                                            R2 := Rect(0, Offset, Width, Height);

                                            Background.Draw(DoorBmp, R1, R2, erAll);
                                            Background.Draw(LightBmp);
                                          end;

                        esRenderingEnd:   begin
                                            fDisplay.WriteLn(sSuccess);
                                            FIsOpen := TRUE;
                                            fMain.Play(PLAY_START2);
                                          end;
                      end;

    //-- Fermeture de la porte -------------------------------------------------

    CLOSEDOOR1:       //Dbut
                      case aEffect.State of
                        esRenderingStart: fDisplay.Write(sClosingSudokuPad);

                        esRenderingRun:   PlayRendering(CLOSEDOOR2, 40);
                      end;

    CLOSEDOOR2:       //La porte coulisse (Remonte)
                      case aEffect.State of
                        esRenderingStart: PlaySound(0, 'SLIDEDOOR');

                        esRenderingRun:   begin
                                            Offset := StepDown(aEffect, Height -40);

                                            R1 := Rect(0, 0, Width, Height -Offset);
                                            R2 := Rect(0, Offset, Width, Height);

                                            Background.Draw(DoorBmp, R1, R2, erAll);
                                            Background.Draw(LightBmp);
                                          end;

                        esRenderingEnd:   begin
                                            Sleep(1000);
                                            PlayRendering(CLOSEDOOR3, 40);
                                            end;
                      end;

    CLOSEDOOR3:       //La porte se remet en place (la lumire blanche s'teind)
                      case aEffect.State of
                        esRenderingStart: begin
                                            Background.Draw(DoorBmp, erAll);
                                            Background.Push;
                                            DoFlush := FALSE;
                                          end;

                        esRenderingRun:   Background.Draw(LightBmp, erAll, StepDown(aEffect));

                        esRenderingEnd:   PlayRendering(CLOSEDOOR4, 5);
                      end;

    CLOSEDOOR4:       //Allume la lumire rouge
                      case aEffect.State of
                        esRenderingRun:   begin
                                            PlaySound(0, 'LOCKDOOR', 0.2);
                                            Background.Draw(RedLightBmp, erAll, StepUp(aEffect));
                                          end;

                        esRenderingEnd:   PlayRendering(CLOSEDOOR5, 10);
                      end;

    CLOSEDOOR5:       //Eteind la lumire rouge - Fin
                      case aEffect.State of
                        esRenderingRun:   Background.Draw(RedLightBmp, erAll, StepDown(aEffect));

                        esRenderingEnd:   begin
                                            fDisplay.WriteLn(sSuccess);
                                            fDisplay.WriteLn(sStandBy, TRUE);
                                            Background.Pop;
                                            FIsOpen := FALSE;
                                          end;
                      end;
  end;

  //Force la nouvelle image  l'cran
  if DoFlush then
    Background.Flush;
end;

procedure TfDoor.FormCreate(Sender: TObject);
begin
  inherited;

  FIsOpen := FALSE;

  BkBmp        := LoadPngToBmp('DOOR_FRAME');
  LightBmp     := LoadPngToBmp('DOOR_LIGHT');
  RedLightBmp  := LoadPngToBmp('DOOR_RED_LIGHT');
  DoorBmp      := LoadPngToBmp('DOOR');

  //Cadre de la porte en image de fond
  Background.SetBitmap(BkBmp);
  //Porte ferme
  PlayRendering(INIT_DOOR);
end;

procedure TfDoor.FormDestroy(Sender: TObject);
begin
  BkBmp.Free;
  LightBmp.Free;
  RedLightBmp.Free;
  DoorBmp.Free;

  inherited;
end;

end.
