//Andnotor, developpez.net Defi N5 Sudoku solver
//------------------------------------------------------------------------------
//Effet du lch de chiffre
//------------------------------------------------------------------------------
unit DropNumber;

interface

uses
  Effects, LayeredImage,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, LayeredForm;

type
  TfDropNumber = class(TfLayeredForm)
    procedure FormDestroy(Sender: TObject);
  private
    MinSize :TPoint;
    NbBmp   :TBitmap;
    WaveBmp :TBitmap;

  protected
    //Traitement des effets visuels et sonores.
    procedure DoEffect(const aEffect :TWMRendering); override;

  public
    Constructor Create(aOwner :TComponent; const aNumber :byte; const aDropRect :TRect); reintroduce;
  end;

var
  fDropNumber: TfDropNumber;

//------------------------------------------------------------------------------
const
  NUMBER_DROP  = 0;
  NUMBER_ERASE = 1;

implementation

uses Main, EffectsConst;

{$R *.dfm}

procedure TfDropNumber.DoEffect(const aEffect :TWMRendering);
var
  R1, R2 :TRect;
  Size   :TPoint;

begin
  case aEffect.ID of
    NUMBER_DROP:      //Le chiffre tombe
                      case aEffect.State of
                        esRenderingRun:   with aEffect, NbBmp do
                                          begin
                                            R1 := Rect(0, 0, Width, Height);
                                            R2 := R1;

                                            Size.X := StepUp(aEffect, (Width -MinSize.X) div 2);
                                            Size.Y := StepUp(aEffect, (Height -MinSize.Y) div 2);

                                            InflateRect(R2, -Size.X, -Size.Y);

                                            Background.Draw(NbBmp, R1, R2, erAll);
                                          end;

                        esRenderingEnd:   begin
                                            PlayRendering(DRAW_WAVE, 20);
                                            fMain.NumberDropped;
                                          end;
                      end;

    NUMBER_ERASE:     //La vague diminue
                      case aEffect.State of
                        esRenderingRun:   begin
                                            R1 := Rect(0, 0, Width, Height);
                                            R2 := R1;

                                            Size.X := StepUp(aEffect, (Width -MinSize.X) div 2);
                                            Size.Y := StepUp(aEffect, (Height -MinSize.Y) div 2);

                                            InflateRect(R2, -Size.X, -Size.Y);

                                            Background.Draw(WaveBmp, R1, R2, erAll, StepUp(aEffect));
                                          end;

                        esRenderingEnd:   Release;
                      end;

    DRAW_WAVE:        //La vague grandit et meurt
                      case aEffect.State of
                        esRenderingRun:   begin
                                            R1 := Rect(0, 0, Width, Height);
                                            R2 := R1;

                                            Size.X := StepDown(aEffect, (Width -MinSize.X) div 2);
                                            Size.Y := StepDown(aEffect, (Height -MinSize.Y) div 2);

                                            InflateRect(R2, -Size.X, -Size.Y);

                                            Background.Draw(WaveBmp, R1, R2, erDest, StepDown(aEffect));
                                          end;

                        esRenderingEnd:   Release;
                      end;
  end;

  //Force la nouvelle image  l'cran
  Background.Flush;
end;

constructor TfDropNumber.Create(aOwner: TComponent; const aNumber: byte; const aDropRect :TRect);
begin
  inherited Create(aOwner);

  with aDropRect do
  begin
    MinSize.X := Right -Left;
    MinSize.Y := Bottom -Top;
  end;

  with CenterRect(aDropRect, BoundsRect) do
  begin
    Self.Left := Left;
    Self.Top  := Top;
  end;

  WaveBmp := LoadPngToBmp(format('WATERWAVE', [aNumber]));

  if aNumber = 99 then
    PlayRendering(NUMBER_ERASE, 15)
  else
  begin
    NbBmp := LoadPngToBmp(format('Nb%d_big', [aNumber]));
    PlayRendering(NUMBER_DROP, 15);
  end;
end;

procedure TfDropNumber.FormDestroy(Sender: TObject);
begin
  NbBmp.Free;
  WaveBmp.Free;
  
  inherited;
end;

end.
