//Andnotor, developpez.net Defi N5 Sudoku solver
//------------------------------------------------------------------------------
//Gestion du son et des effets visuels.
//------------------------------------------------------------------------------
//Les sons taient  la base bass sur la librairie BASS. Mais les conditions du
//dfi ne permettant pas l'utilisation de codes tiers, Une gestion personnelle a
//t ajoute.
//Elle est bien moins performante et contient certaines limitations au niveau des
//types d'encodage des sons (PCM 8 bits 11k non-compress) et  l'utilisation se
//rvle moins fluide que BASS.
//
//Il est cependant toujours possible de commuter sur BASS en dfinissant le code
//de compilation BASS dans les options du projet.

unit Effects;

interface

uses Forms, Classes, SysUtils, Controls, Windows, Messages, Bass, ExtCtrls, Math
     {$IFNDEF BASS}, MMSystem {$ENDIF};


type
//------------------------------------------------------------------------------
  //Les diffrentes tapes de notification de la fiche
  TEffectState = (esPlay,
                  esRenderingStart, esRenderingRun, esRenderingEnd,
                  esSoundStart, esSoundRun, esSoundEnd);

//------------------------------------------------------------------------------
  //Structure des messages reu lors d'animation
  TWMRendering = packed record
    Msg     :Cardinal;
    ID      :byte;
    State   :TEffectState;
    Step    :byte;
    Steps   :byte;
    Param   :LongInt;
  end;

  //Structure des messages reu lors de son
  TWMSound = packed record
    Msg     :Cardinal;
    ID      :byte;
    State   :TEffectState;
    Unused  :word;
    Level   :dword;
  end;



//------------------------------------------------------------------------------
  //Tche ddie  l'affichage
  TRenderingThread = class(TThread)
  private
    Handle   :THandle;                //Handle de la fiche
    ID       :byte;                   //ID de l'vnement demand
    Interval :integer;                //Interval de notification
    Steps    :word;                   //Nombre de pas
    Step     :word;                   //Pas en cours
    Param    :LongInt;                //Paramtre utilisateur

  protected
    //Notifie la fiche de l'avancement
    procedure PostRenderingMessage(const aState :TEffectState);

    //No comment!
    procedure Execute; override;

  public
    //No comment!
    constructor Create(const aHandle: THandle; const aID :byte; const aInterval: integer; const aSteps :word; const aParam: LongInt);
  end;



//------------------------------------------------------------------------------
// Son sans BASS

{$IFNDEF BASS}
  //Entte d'un wav
  TPCMHeader = record
    Tag1       : array[1..4] of char;  { 00..03  Constante "RIFF"       }
    Size1      : LongInt;              { 04..07  Filesize-8             }
    Tag2       : array[1..14] of char; { 08..21  Constante "WAVEfmt..." }
    Mode       : word;                 { 22..23  Mono or Stereo         }
    Freq       : LongInt;              { 24..27  Frequence (Hz)         }
    BytePerSec : LongInt;              { 28..31  Freq*NbrByte           }
    NbrByte    : word;                 { 32..33  (Format div 8)*Mode    }
    Format     : word;                 { 34..35  8 or 16 bits           }
    Tag3       : array[1..4] of char;  { 36..39  Constante "data"       }
    Size2      : LongInt;              { 40..43  Filesize-116           }
  end;

  //Tche ddie aux sons
  TSoundThread = class(TThread)
  private
    Handle     :THandle;           //Handle de la fiche qui recevra les messages
    ID         :byte;              //ID de l'vnement demand
    SoundName  :string;

    Header     :TPCMHeader;        //Entte de la ressource
    WaveOut    :HWaveOut;          //Audio device
    WaveFormat :TWaveFormatEx;     //Format du wav
    WaveHdr    :TWaveHdr;          //Entte des donnes son
    Level      :byte;              //Niveau sonore
    Stream     :TStream;           //Ressource
    Failed     :boolean;           //A chou

  protected
    //Initialisation et lecture
    procedure Open;

    //Libration
    procedure Close;

    //Notifie la fiche de l'avancement
    procedure PostSoundMessage(const aState :TEffectState);

    //No comment!
    procedure Execute; override;

  public
    //No comment!
    constructor Create(const aHandle: THandle; const aID :byte; const aSoundName :string);
    destructor Destroy; override;
  end;
{$ENDIF}



//------------------------------------------------------------------------------
// Son avec BASS

{$IFDEF BASS}

//------------------------------------------------------------------------------
  //Correspond au paramtre user de BASS_ChannelSetSync. User est une structure
  //utilisateur transmise  la procdure CallBack.
  TSoundData = class
  private
    Channel :dword;                   //Canal
    Handle  :THandle;                 //Handle de la fiche qui recevra les messages
    ID      :byte;                    //ID de l'vnement demand
    Level   :word;                    //Niveau sonore
    Stream  :TStream;                 //Ressource
    Timer   :TTimer;

  protected
    //Notifie la fiche de l'avancement
    procedure PostSoundMessage(const aState :TEffectState);

    //No comment!
    procedure OnTimer(Sender: TObject);

  public
    //No comment!
    constructor Create(const aHandle :THandle; const aChannel :dword; const aStream :TStream; const aID :byte);
    destructor  Destroy; override;
  end;

//------------------------------------------------------------------------------
  //Detruire TSoundData depuis SoundEndCallback entrane une AV au retour de
  //la fonction! Plutt que de reporter cette tche sur la fiche concerne
  //(dans le thread principal), j'ai prfr crer une classe grant cela de
  //faon asynchrone.
  TDummy = class
  protected
    Handle :THandle;
    procedure WndProc(var Msg: TMessage);
  public
    constructor Create;
    destructor  Destroy; override;
  end;
{$ENDIF}



//------------------------------------------------------------------------------
const
  //Messages de notification
  WM_EFFECT = WM_USER;

//------------------------------------------------------------------------------
//Joue un son
procedure PlaySound(aHandle :THandle; aID :integer; aSoundName :string; aVolume :extended = 1);
function SoundLength(aSoundName :string) :integer;

//Joue une animation
procedure PlayRendering(aHandle: THandle; aID :integer; aSteps :byte; aParam :LongInt = 0; aInterval :integer = 40);

implementation

//------------------------------------------------------------------------------
const
  //Messages de destruction pour Dummy
  WM_FREESOUNDDATA = WM_USER;

{$IFDEF BASS}
var
  Dummy :TDummy;
{$ENDIF}

//------------------------------------------------------------------------------
{$IFDEF BASS}
procedure SoundEndCallback(aHandle :HSYNC; aChannel :dword; aData :dword; aUser :pointer); stdcall;
begin
  //TSoundData est dtruit depuis Dummy
  PostMessage(Dummy.Handle, WM_FREESOUNDDATA, dword(aUser), 0);
end;
{$ENDIF}

//------------------------------------------------------------------------------
procedure PlaySound(aHandle :THandle; aID :integer; aSoundName :string; aVolume :extended);
var
  Channel :dword;
{$IFDEF BASS}
  Stream  :TResourceStream;
  Data    :TSoundData;
{$ENDIF}

begin
  Assert(FindResource(hInstance, PChar(aSoundName), RT_RCDATA) <> 0, Format('PlaySound: La ressource %s n''existe pas', [aSoundName]));

{$IFDEF BASS}
  //Charge la ressource
  Stream  := TResourceStream.Create(hInstance, aSoundName, RT_RCDATA);

  Channel := BASS_StreamCreateFile(TRUE, Stream.Memory, 0, Stream.Size, 0);

  //Volume
  BASS_ChannelSetAttribute(Channel, BASS_ATTRIB_VOL, Min(aVolume, 1));

  //Synchonisation toute les N ms
  Data := TSoundData.Create(aHandle, Channel, Stream, aID);

  //Synchonisation sur la fin du son
  BASS_ChannelSetSync(Channel, BASS_SYNC_END or BASS_SYNC_ONETIME, 0, SoundEndCallback, Data);

  //Play
  BASS_ChannelPlay(Channel, FALSE);

{$ELSE}
  TSoundThread.Create(aHandle, aID, aSoundName);
{$ENDIF}
end;

//------------------------------------------------------------------------------
function SoundLength(aSoundName :string) :integer;
var
  Channel :dword;
  Stream  :TResourceStream;
{$IFNDEF BASS}
  Header  :TPCMHeader;
{$ENDIF}
begin
  Assert(FindResource(hInstance, PChar(aSoundName), RT_RCDATA) <> 0, Format('SoundLength: La ressource %s n''existe pas', [aSoundName]));

  //Charge la ressource
  Stream  := TResourceStream.Create(hInstance, aSoundName, RT_RCDATA);

{$IFDEF BASS}
  Channel := BASS_StreamCreateFile(TRUE, Stream.Memory, 0, Stream.Size, 0);

  //Dure en ms
  Result := BASS_ChannelGetLength(channel, BASS_POS_BYTE);
  Result := Round(BASS_ChannelBytes2Seconds(channel, Result) *1000);

  //Ferme le canal
  BASS_StreamFree(Channel);

{$ELSE}
  Stream.ReadBuffer(Header, SizeOf(Header));
  Result := (Stream.Size -SizeOf(TPCMHeader)) *1000 div Header.Freq;
{$ENDIF}

  Stream.Free;
end;

//------------------------------------------------------------------------------
procedure PlayRendering(aHandle: THandle; aID :integer; aSteps :byte; aParam :LongInt; aInterval :integer);
begin
  //Cration du thread d'animation
  TRenderingThread.Create(aHandle, aID, aInterval, aSteps, aParam);
end;

{ TSoundData }

{$IFDEF BASS}
//------------------------------------------------------------------------------
constructor TSoundData.Create(const aHandle :THandle; const aChannel :dword; const aStream :TStream; const aID :byte);
begin
  Handle         := aHandle;
  Channel        := aChannel;
  Stream         := aStream;
  ID             := aID;
  Level          := 0;
  Timer          := TTimer.Create(nil);
  Timer.Interval := 40;
  Timer.OnTimer  := OnTimer;

  //Notifie la fiche (Start)
  PostSoundMessage(esSoundStart);
end;

//------------------------------------------------------------------------------
destructor TSoundData.Destroy;
begin
  Timer.Free;
  Stream.Free;
  
  //Notifie la fiche (End)
  PostSoundMessage(esSoundEnd);

  inherited;
end;

//------------------------------------------------------------------------------
procedure TSoundData.OnTimer(Sender: TObject);
var
  LevelLR :dword;
begin
  //Rcupre le niveau sonore depuis BASS
  if (Bass_ChannelisActive(Channel) = BASS_ACTIVE_PLAYING) then
  begin
    LevelLR := BASS_ChannelGetLevel(Channel);

    //Mono niveau
    Level := Max(LoWord(LevelLR), HiWord(LevelLR));

    //Notifie la fiche (Run)
    PostSoundMessage(esSoundRun);
  end;
end;

//------------------------------------------------------------------------------
procedure TSoundData.PostSoundMessage(const aState: TEffectState);
var
  wParam :dword;
begin
  wParam := ID or (dword(aState) shl 8);

  PostMessage(Handle, WM_EFFECT, wParam, Level);
end;
{$ENDIF}

//------------------------------------------------------------------------------

{$IFNDEF BASS}
Function WaveOutThreadProc(aParam :pointer) :LongWord; stdcall;
var
  Msg :TMsg;

begin
  while GetMessage(Msg, 0, 0, 0) do
    case Msg.Message of
      WOM_DONE : //Notification de fin du son
                 begin
                   TThread(aParam).Terminate;
                   ExitThread(ERROR_SUCCESS);
                 end;
    end;
end;

{ TSoundThread }

//------------------------------------------------------------------------------
constructor TSoundThread.Create(const aHandle: THandle; const aID: byte;
  const aSoundName: string);
begin
  inherited Create(FALSE);

  Assert(FindResource(hInstance, PChar(aSoundName), RT_RCDATA) <> 0, Format('TSoundThread.Create: La ressource %s n''existe pas', [aSoundName]));

  Handle := aHandle;
  ID     := aID;
  SoundName := aSoundName;

  FreeOnTerminate := TRUE;
end;

//------------------------------------------------------------------------------
destructor TSoundThread.Destroy;
begin

  inherited;
end;

//------------------------------------------------------------------------------
procedure TSoundThread.Execute;
var
  MMTime :TMMTime;

begin
  //Notification (Start)
  PostSoundMessage(esSoundStart);

  Open;

  while not Terminated do
  begin
    //Position en cours
    waveOutGetPosition(WaveOut, @MMTime, SizeOf(MMTime));

    //Lit dans le stream le niveau sonore (1 octet)
    Stream.Seek(MMTime.cb +SizeOf(Header), soFromBeginning);
    Stream.Read(Level, 1);

    //Notification (Run)
    PostSoundMessage(esSoundRun);

    Sleep(40);
  end;

  Close;

  //Notification (End)
  PostSoundMessage(esSoundEnd);
end;

//------------------------------------------------------------------------------
procedure TSoundThread.Open;
var
  ThreadID :dword;
  hThread  :THandle;
begin
  //Chargement de la ressource et lecture de l'entte
  Stream  := TResourceStream.Create(hInstance, SoundName, RT_RCDATA);
  Stream.ReadBuffer(Header, SizeOf(Header));

  with WaveFormat do
  begin
    wFormatTag      := WAVE_FORMAT_PCM;
    wBitsPerSample  := Header.Format;
    nChannels       := Header.Mode;
    nSamplesPerSec  := Header.Freq;
    nBlockAlign     := nChannels *wBitsPerSample div 8;
    nAvgBytesPerSec := nSamplesPerSec *nBlockAlign;
    cbSize          := 0;
  end;

  //Malgr ce thread de notification, j'ai encore une synchronisation
  //avec le thread principal rendant l'utilisation moins fluide !
  hThread := CreateThread(nil, 0, @WaveOutThreadProc, Self, 0, ThreadID);

  Failed  := (hThread = 0) or
             (WaveOutOpen(@WaveOut,WAVE_MAPPER, @WaveFormat, ThreadID, dword(Self), CALLBACK_THREAD) <> MMSYSERR_NOERROR);

  if not Failed  then
  begin
    Stream.Seek(SizeOf(TPCMHeader), soFromBeginning);

    ZeroMemory(@WaveHdr, sizeOf(TWaveHdr));
    WaveHdr.dwFlags        := WHDR_DONE;

    //-1000 vite (un peu) un scratch en fin de son !!!
    //Il y a encore quelque chose qui m'chappe l-dedans :-)
    WaveHdr.dwBufferLength := Stream.Size -SizeOf(TPCMHeader) -1000;

    GetMem(WaveHdr.lpData, WaveHdr.dwBufferLength);

    Stream.Read(WaveHdr.lpData^, WaveHdr.dwBufferLength);

    WaveOutPrepareHeader(WaveOut, @WaveHdr, SizeOf(TWaveHdr));
    waveOutWrite(WaveOut, @WaveHdr, SizeOf(WaveHdr));
  end
  else Terminate;
end;

//------------------------------------------------------------------------------
procedure TSoundThread.Close;
begin
  if not Failed then
  begin
    WaveOutUnprepareHeader(WaveOut, @WaveHdr, sizeof(TWaveHdr));
    WaveOutClose(WaveOut);
    FreeMem(WaveHdr.lpData);
  end;

  Stream.Free;
end;

//------------------------------------------------------------------------------
procedure TSoundThread.PostSoundMessage(const aState: TEffectState);
var
  wParam :dword;
begin
  wParam := ID or (dword(aState) shl 8);

  PostMessage(Handle, WM_EFFECT, wParam, Level);
end;
{$ENDIF}

{ TRenderingThread }

//------------------------------------------------------------------------------
constructor TRenderingThread.Create(const aHandle: THandle; const aID :byte;
  const aInterval: integer; const aSteps :word; const aParam: integer);
begin
  inherited Create(FALSE);

  Handle   := aHandle;
  ID       := aID;
  Interval := aInterval;
  Step     := 1;
  Steps    := aSteps;
  Param    := aParam;

  FreeOnTerminate := TRUE;
end;

//------------------------------------------------------------------------------
procedure TRenderingThread.Execute;
begin
  //Notifie la fiche (Start)
  PostRenderingMessage(esRenderingStart);

  while not Terminated and (Step <= Steps) do
  begin
    //Notifie la fiche (Run)
    PostRenderingMessage(esRenderingRun);

    //Pause
    Sleep(Interval);

    inc(Step);
  end;

  //Notifie la fiche (End)
  PostRenderingMessage(esRenderingEnd);
end;

//------------------------------------------------------------------------------
procedure TRenderingThread.PostRenderingMessage(const aState: TEffectState);
var
  wParam :dword;
begin
  wParam := ID                     or
            (dword(aState) shl 8)  or
            (dword(Step)   shl 16) or
            (dword(Steps)  shl 24);

  PostMessage(Handle, WM_EFFECT, wParam, Param);
end;

{ TDummy }

{$IFDEF BASS}
//------------------------------------------------------------------------------
constructor TDummy.Create;
begin
  //Cration d'une fentre
  Handle := AllocateHWnd(WndProc);
end;

//------------------------------------------------------------------------------
destructor TDummy.Destroy;
begin
  //Liberation de la fentre
  DeallocateHWnd(Handle);
  inherited;
end;

//------------------------------------------------------------------------------
procedure TDummy.WndProc(var Msg: TMessage);
begin
  if Msg.Msg = WM_FREESOUNDDATA then
    with TSoundData(Msg.WParam) do
    begin
      //Libration du canal
      BASS_StreamFree(Channel);

      //Libration du TSoundData
      Free;
    end;
end;
{$ENDIF}

//------------------------------------------------------------------------------

initialization
{$IFDEF BASS}
  BASS_Init(-1, 11000, 0, Application.Handle, nil);
  Dummy := TDummy.Create;
{$ENDIF}

finalization
{$IFDEF BASS}
  Dummy.Free;
{$ENDIF}

end.
