//Andnotor, developpez.net Defi N5 Sudoku solver
//------------------------------------------------------------------------------
//HAL 9000 (Voir aussi TfLayeredForm)
//------------------------------------------------------------------------------
unit HAL;

interface

uses
  Effects, LayeredImage,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, LayeredForm, Math;

type
//------------------------------------------------------------------------------
  TfHAL = class(TfLayeredForm)
    procedure FormDestroy(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDblClick(Sender: TObject);

  private
    HALBmp        :TBitmap;              //HAL
    HALOFFBmp     :TBitmap;              //HAL teind
    HALTalkingBmp :TBitmap;              //Effet sur voie
    StarBmp       :TBitmap;              //Etoile affiche  la fin du programme

  protected
    //Traitement des effets visuels et sonores.
    procedure DoEffect(const aEffect :TWMRendering); override;
  end;

var
  fHAL: TfHAL;

implementation

uses EffectsConst, Display, Main;

//------------------------------------------------------------------------------
//Constantes bases sur les constatations faites en coutant les diffrents sons.
const
  //Niveau sonore au dessous duquel les son ne contient que du bruit ambiant
{$IFDEF BASS}
  MinLevel  = 8000;
{$ELSE}
  MinLevel  = 130;
{$ENDIF}

  //Niveau sonore concidr comme maximum
{$IFDEF BASS}
  MaxLevel  = 13000;
{$ELSE}
  MaxLevel  = 170;
{$ENDIF}

  //Fluctuation de la voie
{$IFDEF BASS}
  LevelStep = 255 /MaxLevel;
{$ELSE}
  LevelStep = 255 /MaxLevel;
{$ENDIF}

{$R *.dfm}

{ TfLayeredForm3 }

//------------------------------------------------------------------------------
procedure TfHAL.DoEffect(const aEffect :TWMRendering);
var
  //Permet d'inteprter aEffect comme un TWMSound sans transtypage
  Sound :TWMSound absolute aEffect;

  i :integer;
begin
  inherited;

  if aEffect.State = esPlay then
  begin
    case aEffect.ID of
      HAL_ON                    : PlayRendering(aEffect.ID, 100);
      HAL_TALKING_HELLODAVE     : PlaySound(aEffect.ID, 'HELLO_DAVE');
      HAL_TALKING_JUSTONEMOMENT : PlaySound(aEffect.ID, 'JUST_ONE_MOMENT');
      HAL_TALKING_INFOCOMPLETED : PlaySound(aEffect.ID, 'INFO_COMPLETED');
      HAL_TALKING_ICANNOTDOTHAT : PlaySound(aEffect.ID, 'I_CANNOT_DO_THAT');
      HAL_TALKING_JUSTWHAT      : PlaySound(aEffect.ID, 'JUST_WHAT');
      HAL_TALKING_STOPDAVE      : PlaySound(aEffect.ID, 'STOP_DAVE');
    end;

    Exit;
  end;

  Background.Erase;

  case aEffect.ID of
    //HAL parle...
    HAL_TALKING_FIRST..
    HAL_TALKING_LAST :  case Sound.State of
                          esSoundStart: begin
                                          case Sound.ID of
                                            HAL_TALKING_DAISY:         begin
                                                                         i := SoundLength('DAISY') div 255;
                                                                         PlayRendering(HAL_FADINGOUT, 255, 0, i);
                                                                       end;
                                          end;
                                        end;

                          esSoundRun:   //"Mouvement des lvres" de HAL
                                        begin
                                          SoundLevel := Sound.Level;

                                          if SoundLevel < MinLevel then
                                            SoundLevel := 0
                                          else
                                          begin
                                            SoundLevel := Min(Trunc(SoundLevel *LevelStep), 255);
                                            Background.Draw(HALTalkingBmp, erAll, SoundLevel);
                                          end;
                                        end;

                         esSoundEnd:    begin
                                          SoundLevel := 0;

                                          //Enchanement de l'action suivante
                                          case Sound.ID of
                                            HAL_TALKING_STOPDAVE:      begin
//                                                                         Sleep(3000);
//                                                                         i := (SoundLength('MY_MIND_IS_GOING') + SoundLength('GOOD_AFTERNOON')) div 255;
//                                                                         fMain.Play(PLAY_QUIT2, i);
                                                                         PlaySound(HAL_TALKING_MYMINDISGOING, 'MY_MIND_IS_GOING');
                                                                       end;

                                            HAL_TALKING_MYMINDISGOING: begin
                                                                         i := (SoundLength('GOOD_AFTERNOON') + SoundLength('DAISY')) div 255;
                                                                         PlaySound(HAL_TALKING_GOODAFTERNOON, 'GOOD_AFTERNOON');
                                                                         fMain.Play(PLAY_QUIT2, i);
                                                                       end;

                                            HAL_TALKING_GOODAFTERNOON: PlaySound(HAL_TALKING_DAISY, 'DAISY');

                                            HAL_TALKING_DAISY:         begin
                                                                         Sleep(2000);
                                                                         PlayRendering(STAR_FADINGIN, 15, 10);
                                                                       end;
                                          end;
                                        end;
                        end;

    HAL_ON:             case aEffect.State of
                          esRenderingStart: begin
                                              fDisplay.WriteLn(sPoweringOn, TRUE);
                                            end;

                          esRenderingRun:   Background.Draw(HALOFFBmp, erALL, StepDown(aEffect));

                          esRenderingEnd:   begin
                                              Play(HAL_TALKING_HELLODAVE);
                                              fDisplay.WriteLn(sHelloDave, TRUE);
                                            end;
                        end;

    //HAL disparait (Fin du programme)
    HAL_FADINGOUT:      case aEffect.State of
                          esRenderingRun:   Background.Alpha := StepDown(aEffect);
                        end;

    //L'toile apparait (Fin du programme)
    STAR_FADINGIN:      case aEffect.State of
                          esRenderingStart: begin
                                              //Modifie l'image de fond
                                              Background.SetBitmap(StarBmp);

                                              //100% transparente
                                              Background.Alpha  := 0;

                                              Background.Erase;
                                            end;

                          esRenderingRun:   //L'toile apparait gentillement
                                            Background.Alpha := StepUp(aEffect);

                          esRenderingEnd:   //La refait disparaitre
                                            PlayRendering(STAR_FADINGOUT, 50, 10);
                        end;


    //L'toile disparait (Fin du programme)
    STAR_FADINGOUT :    case aEffect.State of
                          esRenderingRun:   //L'toile disparait gentillement
                                            Background.Alpha := StepDown(aEffect);

                          esRenderingEnd:   begin
                                              //Fin du programme
                                              Sleep(2000);
//                                              PostQuitMessage(0);
                                              Application.Terminate;
                                            end;
                        end;

  end;

  //Force la nouvelle image  l'cran
  Background.Flush;
end;

procedure TfHAL.FormCreate(Sender: TObject);
begin
  inherited;

  //Images
  HALBmp        := LoadPngToBmp('HAL');
  HALTalkingBmp := LoadPngToBmp('HAL_TALKING');
  HALOFFBmp     := LoadPngToBmp('HAL_OFF');
  StarBmp       := LoadPngToBmp('STAR');

  //Fixe HAL comme image de fond
  Background.SetBitmap(HALBmp);
  Background.Draw(HALOFFBmp);
  Background.Flush;
end;

procedure TfHAL.FormDblClick(Sender: TObject);
begin
  inherited;
  Application.Terminate;
end;

procedure TfHAL.FormDestroy(Sender: TObject);
begin
  //Libration des images
  HALBmp.Free;
  HALTalkingBmp.Free;
  HALOFFBmp.Free;
  StarBmp.Free;

  inherited;
end;

end.
