//Andnotor, developpez.net Defi N5 Sudoku solver
//------------------------------------------------------------------------------
//Fiche de base implmentant le comportement des fiches layered.
//------------------------------------------------------------------------------
unit LayeredForm;

interface

uses
  LayeredImage, Effects,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Math;

type
  //----------------------------------------------------------------------------
  TfLayeredForm = class(TForm)
    //No comment!
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);

  private
    //Messages reus depuis l'unit Effects
    procedure WMEffect(var Message :TWMRendering); message WM_EFFECT;

    //Renvoi le bitmap interne
    function GetBitmap: TBitmap;

  protected
    Background :TLayeredImage;      //Gestionnaire de fiche layered
    SoundLevel :dword;              //Niveau sonore

    //Procdure servant  la gestion des messages animations et sons.
    procedure DoEffect(const aEffect :TWMRendering); virtual;

    //Joue un son
    procedure PlaySound(aID :integer; aSoundName :string; aVolume :extended = 1);

    //Joue une animation
    procedure PlayRendering(aID :integer; aSteps :byte = 1; aParam :LongInt = 0; aInterval :integer = 40);

  public
    property  Bitmap :TBitmap read GetBitmap;

    //Renvoi un pourcentage de aValue
    function  StepDown(const aEffect :TWMRendering; const aValue :integer = 255) :integer;
    function  StepUp(const aEffect :TWMRendering; const aValue :integer = 255) :integer;

    //Centre un rectangle par rapport  un autre
    function  CenterRect(const aBaseRect, aMovedRect :TRect) :TRect;

    //Joue un son ou une animation
    procedure Play(aID :integer; aParam :integer = 0);
  end;

implementation

uses EffectsConst;

{$R *.dfm}

//------------------------------------------------------------------------------
function TfLayeredForm.CenterRect(const aBaseRect, aMovedRect: TRect): TRect;
var
  BaseCenter  :TPoint;
  MovedCenter :TPoint;
begin
  with aBaseRect do
  begin
    BaseCenter.X := Left +(Right -Left) div 2;
    BaseCenter.Y := Top +(Bottom -Top) div 2;
  end;

  with aMovedRect do
  begin
    MovedCenter.X := Left +(Right -Left) div 2;
    MovedCenter.Y := Top +(Bottom -Top) div 2;
  end;

  Result := aMovedRect;
  OffsetRect(Result, BaseCenter.X -MovedCenter.X, BaseCenter.Y -MovedCenter.Y);
end;

procedure TfLayeredForm.DoEffect(const aEffect: TWMRendering);
begin
  case aEffect.ID of
    //Beep
    PLAY_BEEP : PlaySound(0, 'BEEP');
  end;
end;

procedure TfLayeredForm.FormCreate(Sender: TObject);
begin
  //Cration du gestionnaire de fiche layered
  BackGround := TLayeredImage.Create(Self);
end;

procedure TfLayeredForm.FormDestroy(Sender: TObject);
begin
  //Libration
  BackGround.Free;
end;

function TfLayeredForm.GetBitmap: TBitmap;
begin
  Result := Background.InternalBmp;
end;

procedure TfLayeredForm.Play(aID: integer; aParam :integer);
var
  wParam :dword;
begin
  wParam := aID or (dword(esPlay) shl 8);

  PostMessage(Handle, WM_EFFECT, wParam, aParam);
end;

procedure TfLayeredForm.PlayRendering(aID: integer; aSteps: byte; aParam :LongInt; aInterval: integer);
begin
  //Joue une animation
  Effects.PlayRendering(Handle, aID, aSteps, aParam, aInterval);
end;

procedure TfLayeredForm.PlaySound(aID: integer; aSoundName: string; aVolume :extended);
begin
  //Joue un son
  Effects.PlaySound(Handle, aID, aSoundName, aVolume);
end;

function TfLayeredForm.StepDown(const aEffect: TWMRendering; const aValue: integer): integer;
begin
  Result := aValue -StepUp(aEffect, aValue);
end;

function TfLayeredForm.StepUp(const aEffect: TWMRendering; const aValue: Integer): integer;
begin
  Result := Round(aValue /aEffect.Steps *aEffect.Step);
end;

procedure TfLayeredForm.WMEffect(var Message: TWMRendering);
begin
  DoEffect(Message);
end;

end.
