//Andnotor, developpez.net Defi N5 Sudoku solver
//------------------------------------------------------------------------------
//Fiche principale (Voir aussi TfLayeredForm)
//------------------------------------------------------------------------------
unit Main;

interface

uses
  Effects, LayeredImage, uSudoku,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, LayeredForm, StdCtrls, Buttons, Grids, pngimage, ExtCtrls, Math,
  AppEvnts,
  //SpeedButtonEx doit obligatoirement tre en dernier!
  SpeedButtonEx;

const
  WM_AFTERSHOW = WM_USER +1000;

type
  //----------------------------------------------------------------------------

  //Rsultat de l'ajout d'un chiffre
  TDropAccepted = (daNone, daErase, daNew);

  //Grille temporaire pour affichage
  TResolvedGrid = array of TPoint;

//------------------------------------------------------------------------------
  TfMain = class(TfLayeredForm)
    Grid: TStringGrid;
    bOff: TSpeedButton;
    bPlay: TSpeedButton;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    ApplicationEvents1: TApplicationEvents;
    procedure FormCreate(Sender: TObject);
    procedure bOffClick(Sender: TObject);
    procedure GridClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure GridMouseEnter(Sender: TObject);
    procedure ApplicationEvents1Deactivate(Sender: TObject);
    procedure ApplicationEvents1Activate(Sender: TObject);
    procedure bOffMouseEnter(Sender: TObject);
    procedure bPlayClick(Sender: TObject);
    procedure FormShow(Sender: TObject);

  private
    Sudoku            :TSudoku;      //Le sudoku solver

    FDroppingNumber   :byte;         //Chiffre en cours d'ajout

    BkBmp             :TBitmap;      //Image de fond
    BtnBmp            :TBitmap;      //Image d'un bouton
    PurpleBtnBmp      :TBitmap;      //Couleur violette du bouton dsactiv
    PurpleShadeBtnBmp :TBitmap;      //Couleur violette du bouton activ

    DropAccepted    :TDropAccepted;  //Rsultat de l'ajout d'un chiffre

    ResolvedGrid    :TResolvedGrid;  //Grille temporaire pour le rendu  l'cran

    //Application.NormalizeTopMosts ne fonctionne pas (Bug ?!?)
    procedure NormalizeTopMostsEx;
    procedure RestoreTopMostsEx;

  protected
    //Traitement des effets visuels et sonores.
    procedure DoEffect(const aEffect :TWMRendering); override;

    //Dessin d'un chiffre
    procedure DrawNumber(const aCol, aRow, aValue :byte; const aResolved :boolean; const aOpacity :byte = 255);

    //Rcupre l'image d'une autre fiche et l'ajoute au fond
    procedure Draw(const aForm :TfLayeredForm);

    procedure WMAfterShow(var Message :TMessage); message WM_AFTERSHOW;

  public
    property  DroppingNumber :byte read FDroppingNumber write FDroppingNumber;

    //Le chiffre est arriv au "sol"
    procedure NumberDropped;

    //Vide la grille (Bouton New)
    procedure New;

    //Charge une grille (Bouton Load)
    procedure Load;

    //Sauve la grille (Bouton Save)
    procedure Save;

    //Lance la rsolution (Bouton Ignation)
    procedure Solve;
  end;

var
  fMain: TfMain;

//------------------------------------------------------------------------------

implementation

uses EffectsConst, HAL, Door, NumberButtons, CommandButtons, DropNumber, Display;

const
  //Niveau sonore au dessous duquel les son ne contient que du bruit ambiant
{$IFDEF BASS}
  MaxVibration = 10000;
{$ELSE}
  MaxVibration = 110;
{$ENDIF}

{$R *.dfm}

procedure TfMain.bPlayClick(Sender: TObject);
begin
  inherited;
  fHAL.Play(HAL_TALKING_JUSTONEMOMENT);

  if fDoor.IsOpen
  then Play(PLAY_STOP)
  else Play(PLAY_START);
end;

procedure TfMain.DoEffect(const aEffect: TWMRendering);
var
  //Permet d'inteprter aEffect comme un TWMSound sans transtypage
  Sound :TWMSound absolute aEffect;

  Bmp :TBitmap;
  Pos :TPoint;

  R1, R2 :TRect;
  i      :integer;
  Number :byte;
begin
  inherited;

  if aEffect.State = esPlay then
  begin
    case aEffect.ID of
      DROPNUMBER_OK   : //** Chiffre ajout avec succs **
                        //- Dtonation
                        //- Vibrations synchronises sur la dtonation
                        PlaySound(aEffect.ID, 'EXPLOSION');

      DROPNUMBER_FAIL : //** Chiffre refus **
                        //- Plouf
                        //- HAL "I cannot do that"
                        PlaySound(aEffect.ID, 'PLOUF');

      DRAW_BUTTON     : //** Dessine un bouton **
                        //en lui passant en rfrence le bouton concern
                        PlayRendering(aEffect.ID, 1, aEffect.Param);
      PLAY_START,
      PLAY_START2     : //** Active le jeu ***
                        //- Ouvre la porte du pad
                        //  <synchro>
                        //- Active les boutons
                        PlayRendering(aEffect.ID);
      
      PLAY_STOP,
      PLAY_STOP2      : //** Dsactive le jeu ***
                        //- Dsactive les boutons
                        //  <synchro>
                        //- Ferme la porte du pad
                        PlayRendering(aEffect.ID);

      PLAY_QUIT       : //** Fin du jeu 1 **
                        //- Ferme la porte du pad si ouverte
                        //- HAL "Stop Dave", "My mind is going"
                        //  <synchro>
                        //- PLAY_QUIT2
                        PlayRendering(aEffect.ID);

      PLAY_QUIT2      : //** Fin du jeu 2 **
                        //- Copie les images des fiches, les masque
                        //- La fiche principale disparait gentillement
                        PlayRendering(aEffect.ID, 255, 0, aEffect.Param);

      SUDOKU_RESOLVED : //Affiche les numros en se synchronisant sur la dure du son
                        begin
                          if Length(ResolvedGrid) > 0 then
                          begin
                            fHAL.Play(HAL_TALKING_JUSTONEMOMENT);
                            PlaySound(aEffect.ID, 'PROCESSING');
                            i := SoundLength('PROCESSING') div Length(ResolvedGrid);
                          end
                          else i := 1;

                          PlayRendering(aEffect.ID, Length(ResolvedGrid), 0, i);
                        end;

      SUDOKU_FAIL     : //La rsolution de la grille a choue
                        PlayRendering(aEffect.ID);
    end;

    Exit;
  end;

  case aEffect.ID of

    //-- Dessin de la grille ---------------------------------------------------

    DRAW_GRID:   case aEffect.State of
                          esRenderingRun:   begin
                                              Bmp := LoadPngToBmp('GRID');
                                              R1 := Rect(0, 0, Bmp.Width, Bmp.Height);
                                              Background.Draw(Bmp, R1, Grid.BoundsRect);
                                              Bmp.Free;
                                            end;
                        end;

    //-- Activation du jeu -----------------------------------------------------

    PLAY_START1:        case aEffect.State of
                          esRenderingRun:   //Ouverture de la porte
                                            fDoor.Play(OPENDOOR);
                        end;

    PLAY_START2:        case aEffect.State of
                          esRenderingStart: Sleep(1000);

                          esRenderingRun:   //Activation des boutons
                                            begin
                                              fNumberButtons.Play(PLAY_START);
                                              fCommandButtons.Play(PLAY_START);
                                            end;
                        end;

    //-- Dsactivation du jeu --------------------------------------------------

    PLAY_STOP1:         case aEffect.State of
                          esRenderingRun:   //Dsactivation des boutons
                                            begin
                                              fNumberButtons.Play(PLAY_STOP);
                                              fCommandButtons.Play(PLAY_STOP);
                                            end;
                        end;

    PLAY_STOP2:         case aEffect.State of
                          esRenderingStart: Sleep(1000);

                          esRenderingRun:   //Fermeture de la porte
                                            fDoor.Play(CLOSEDOOR);
                        end;

    //-- Quitte l'application --------------------------------------------------

    PLAY_QUIT1:         case aEffect.State of
                          esRenderingStart: begin
                                              PlaySound(0, 'SHUTDOWN');
                                              Sleep(1000);
                                            end;

                          esRenderingRun:   //Stop Dave et ferme la porte
                                            begin
                                              fHAL.Play(HAL_TALKING_STOPDAVE);
                                              if fDoor.IsOpen then
                                                Play(PLAY_STOP);
                                            end;
                        end;

    PLAY_QUIT2:         case aEffect.State of
                          esRenderingStart: //Copie les image et cache les fiches
                                            begin
                                              Draw(fDoor);
                                              Draw(fNumberButtons);
                                              Draw(fCommandButtons);
                                              fDisplay.Hide;
                                              Background.Push;
                                              
                                              Background.Flush;

                                              fDoor.Hide;
                                              fNumberButtons.Hide;
                                              fCommandButtons.Hide;
                                              fDisplay.Hide;
                                            end;

                          esRenderingRun:   //La fiche principale disparait
                                            begin
                                              Background.Alpha := StepDown(aEffect);
                                              Background.Erase;
                                            end;
                        end;

    //-- Chiffre ajout avec succs --------------------------------------------

    DROPNUMBER_OK:      case Sound.State of
                          esSoundStart:     //Dessin du chiffre
                                            begin
                                              PlayRendering(DRAW_NUMBER, 1, MakeLong(Grid.Col, Grid.Row));
                                              fDisplay.WriteLn(Format(sDroppingNumber +sSuccess, [FDroppingNumber]));
                                            end;

                          esSoundRun:       //Vibration
                                            begin
                                              Pos := BoundsRect.TopLeft;
                                              Left := Pos.X +Sound.Level div MaxVibration;
                                              Top  := Pos.Y +Sound.Level div MaxVibration;
                                              Sleep(20);
                                              Left := Pos.X;
                                              Top  := Pos.Y;
                                            end;
                        end;

    //-- Chiffre refus --------------------------------------------------------

    DROPNUMBER_FAIL:     case Sound.State of
                          esSoundStart:     begin
                                              fHAL.Play(HAL_TALKING_ICANNOTDOTHAT);
                                              fDisplay.WriteLn(Format(sDroppingNumber +sFailed, [FDroppingNumber]));
                                            end;
                        end;

    //-- Case efface ----------------------------------------------------------

    DROPNUMBER_ERASE:   case aEffect.State of
                          esRenderingStart: fDisplay.WriteLn(sCellCleared);

                          esRenderingRun:   begin
                                              Number := StrToInt(Grid.Cells[LoWord(aEffect.Param), HiWord(aEffect.Param)]);
                                              DrawNumber(LoWord(aEffect.Param), HiWord(aEffect.Param), Number, FALSE, StepDown(aEffect));
                                            end;

                          esRenderingEnd:   begin
                                              Grid.Cells[LoWord(aEffect.Param), HiWord(aEffect.Param)] := '';
                                              Sudoku.ClearValue(LoWord(aEffect.Param), HiWord(aEffect.Param));
                                              PlaySound(0, 'SLAP');
                                            end;

                        end;

    //-- Dessin du chiffre -----------------------------------------------------

    DRAW_NUMBER:        case aEffect.State of
                          esRenderingRun:   DrawNumber(LoWord(aEffect.Param), HiWord(aEffect.Param), DroppingNumber, FALSE);
                        end;

    //-- Sudoku rsolu ---------------------------------------------------------

    SUDOKU_RESOLVED:    case aEffect.State of
                          esRenderingRun:   //Dessine le chiffre
                                            with ResolvedGrid[aEffect.Step -1] do
                                              DrawNumber(X, Y, StrToInt(Grid.Cells[X, Y]), TRUE);

                          esRenderingEnd:   //Informations de rsolution
                                            begin
                                              fDisplay.WriteLn(Format(sSetValueCount, [Sudoku.SetValueCount]));
                                              fDisplay.WriteTime(sProcessingTime, Sudoku.Duration);
                                              fDisplay.WriteLn(sInfoCompleted);

                                              fHAL.Play(HAL_TALKING_INFOCOMPLETED);
                                              fCommandButtons.EnableButtons(TRUE);
                                              Grid.Enabled := TRUE;
                                            end;
                        end;

    //-- La rsolution  choue -----------------------------------------------

      SUDOKU_FAIL     : case aEffect.State of
                          esRenderingRun:   begin
                                              fHAL.Play(HAL_TALKING_ICANNOTDOTHAT);
                                              fDisplay.WriteLn(sSudokuFail, TRUE);
                                              Grid.Enabled := TRUE;
                                              fNumberButtons.EnableButtons(TRUE);
                                              fCommandButtons.EnableButtons(TRUE);
                                            end;
                        end;

    //-- Dessine les boutons dsactivs ----------------------------------------

    DRAW_BUTTONS:       case aEffect.State of
                          esRenderingRun:   begin
                                              for i := 0 to ControlCount -1 do
                                                if Controls[i] is TSpeedButton then
                                                begin
                                                  R1 := Rect(0, 0, BtnBmp.Width, BtnBmp.Height);
                                                  R2 := Controls[i].BoundsRect;

                                                  Background.Draw(BtnBmp, R1, R2);
                                                end;
                                            end;

                          esRenderingEnd:   //Active les boutons
                                            begin
                                              Background.Push;
                                              bOFF.Enabled  := TRUE;
                                              bPlay.Enabled := TRUE;
                                            end;

                        end;

    //-- Dessine un bouton (ds)activ -----------------------------------------

    DRAW_BUTTON:        case aEffect.State of
                          esRenderingRun:   with TSpeedButton(aEffect.Param) do
                                            begin
                                              if Enabled then
                                              begin
                                                if Down
                                                then Bmp := PurpleShadeBtnBmp
                                                else Bmp := PurpleBtnBmp;

                                                R1 := Rect(0, 0, Bmp.Width, Bmp.Height);
                                                R2 := CenterRect(BoundsRect, R1);

                                                Background.Draw(Bmp, R1, R2, erDest, 150)
                                              end
                                              else
                                              begin
                                                R1 := Rect(0, 0, PurpleShadeBtnBmp.Width, PurpleShadeBtnBmp.Height);
                                                R1 := CenterRect(BoundsRect, R1);
                                                Background.Erase(R1);
                                              end;
                                            end;
                        end;
  end;

  //Force la nouvelle image  l'cran
  Background.Flush;
end;

procedure TfMain.Draw(const aForm: TfLayeredForm);
var
  R1, R2 :TRect;
begin
  R1 := Rect(0, 0, aForm.Width, aForm.Height);
  R2 := R1;

  OffsetRect(R2, aForm.Left -Left, aForm.Top -Top);
  Background.Draw(aForm.Bitmap, R1, R2);
end;

procedure TfMain.DrawNumber(const aCol, aRow, aValue: byte; const aResolved :boolean; const aOpacity :byte);
var
  Bmp    :TBitmap;
  R1, R2 :TRect;
begin
  //Chiffre vert ou blanc ?
  if aResolved
  then Bmp := LoadPngToBmp(Format('NB%d_WHITE', [aValue]))
  else Bmp := LoadPngToBmp(Format('NB%d_GREEN', [aValue]));

  //Centr dans la case
  R1  := Rect(0, 0, Bmp.Width, Bmp.Height);
  R2  := Grid.CellRect(aCol, aRow);
  OffsetRect(R2, Grid.Left, Grid.Top);

  //Dessin
  Background.Draw(Bmp, R1, R2, erDest, aOpacity);
  
  Bmp.Free;
end;

procedure TfMain.GridClick(Sender: TObject);
var
  Current :byte;

begin
  inherited;

  //Chiffre ajout
  if Grid.Enabled then
  begin
    DropAccepted := daNone;

    if FDroppingNumber > 0 then
    begin
      Play(PLAY_BEEP);

      with Grid do
      begin
        //Numro actuel dans la cellule (0 si vide)
        Current := StrToIntDef(Cells[Col, Row], 0);

        if FDroppingNumber <> Current then
        begin
          case FDroppingNumber of
            1..9 :  //Numro accept ?
                    if Sudoku.SetValue(Col, Row, FDroppingNumber) then
                    begin
                      Cells[Col, Row] := IntToStr(FDroppingNumber);
                      DropAccepted    := daNew;
                    end;

            99   :  //Case efface
                    if Cells[Col, Row] <> '' then
                    begin
//                      Sudoku.ClearValue(Col, Row);
                      PlayRendering(DROPNUMBER_ERASE, 15, MakeLong(Col, Row));
                      DropAccepted := daErase;
                    end
                    else Exit;
          end;
        end;

        //Cre TfDropNumber et lui passe les coordonnes (cran) de la cellule
        with ClientToScreen(CellRect(Col, Row).TopLeft) do
          TfDropNumber.Create(Self, DroppingNumber, Rect(X, Y, X +DefaultColWidth, Y +DefaultRowHeight));
      end;
    end;
  end;
end;

procedure TfMain.GridMouseEnter(Sender: TObject);
begin
  inherited;

  case DroppingNumber of
    0  : //Aucune action slectionne
         Grid.Cursor := crNoDrop;

    99 : //Effacement
         Grid.Cursor := crDrag;

    else //Chiffres
         Grid.Cursor := crCross;
  end;
end;

procedure TfMain.Load;
var
  sl  :TStringList;
  Col :byte;
  Row :byte;
  Valid :boolean;

begin
  NormalizeTopMostsEx;

  if OpenDialog1.Execute then
  try
    New;

    sl := TStringList.Create;
    sl.LoadFromFile(OpenDialog1.FileName);

    //Contrle la validit du fichier

    //9 lignes ?
    Valid := sl.Count = 9;
    
    if not Valid then Exit;

    for Row := 0 to sl.Count -1 do
    begin
      //9 chiffres ?
      Valid := (Length(sl[Row]) = 9) and
               (StrToIntDef(sl[Row], -1) <> -1);

      if not Valid then Exit;

      for Col := 0 to 8 do
      begin
        if sl[Row][Col+1] = '0' then
          Grid.Cells[Col, Row] := ''
        else
        begin
          DroppingNumber := StrToInt(sl[Row][Col+1]);

          //Accept ?
          Valid := Sudoku.SetValue(Col, Row, DroppingNumber);

          if not Valid then Exit;

          Grid.Cells[Col, Row] := sl[Row][Col+1];
          DrawNumber(Col, Row, DroppingNumber, FALSE);
        end;
      end;
    end;

  finally
    sl.Free;

    if Valid then
    begin
      fDisplay.WriteLn(Format(sFileLoaded, [ExtractFileName(OpenDialog1.FileName)]));
      Background.Flush;
    end
    else
    begin
      fDisplay.WriteLn(Format(sNotValidFile, [ExtractFileName(OpenDialog1.FileName)]));
      New;
    end;

    RestoreTopMostsEx;
  end;
end;

procedure TfMain.New;
var
  Col :byte;
  Row :byte;
begin
  //Efface la grille et le sudoku
  
  fDisplay.WriteLn(sNewGrid);

  Sudoku.Reset;
  
  for Row := 0 to Grid.RowCount -1 do
    for Col := 0 to Grid.ColCount -1 do
      Grid.Cells[Col, Row] := '';

  Background.Erase(Grid.BoundsRect, TRUE);

  fNumberButtons.EnableButtons(TRUE);
end;

procedure TfMain.NumberDropped;
begin
  case DropAccepted of
    daNone:  Play(DROPNUMBER_FAIL);
    daErase: Play(DROPNUMBER_ERASE);
    daNew:   Play(DROPNUMBER_OK);
  end;
end;

procedure TfMain.NormalizeTopMostsEx;
begin
  fHAL.FormStyle            := fsNormal;
  fDoor.FormStyle           := fsNormal;
  fDisplay.FormStyle        := fsNormal;
  fNumberButtons.FormStyle  := fsNormal;
  fCommandButtons.FormStyle := fsNormal;
end;

procedure TfMain.RestoreTopMostsEx;
begin
  fHAL.FormStyle            := fsStayOnTop;
  fDoor.FormStyle           := fsStayOnTop;
  fDisplay.FormStyle        := fsStayOnTop;
  fNumberButtons.FormStyle  := fsStayOnTop;
  fCommandButtons.FormStyle := fsStayOnTop;
end;

procedure TfMain.Save;
var
  Col :byte;
  Row :byte;
  s   :string;
  sl  :TStringList;
  
begin
  //Sauve la grille
  if SaveDialog1.Execute then
  begin
    sl := TStringList.Create;

    for Row := 0 to Grid.RowCount -1 do
    begin
      s := '';

      for Col := 0 to Grid.ColCount -1 do
        if Grid.Cells[Col, Row] = ''
        then s := s +'0'
        else s := s +Grid.Cells[Col, Row];

      sl.Add(s);
    end;

    try
      sl.SaveToFile(SaveDialog1.FileName);
      fDisplay.WriteLn(Format(sFileSaved, [ExtractFileName(SaveDialog1.FileName)]));
    except
      fDisplay.WriteLn(Format(sFileNotSaved, [ExtractFileName(SaveDialog1.FileName)]));
    end;

    sl.Free;
  end;
end;

procedure TfMain.Solve;
var
  Col :byte;
  Row :byte;
  Pt  :TPoint;
  i, i1, i2 :integer;
  
begin
  fDisplay.WriteLn(sStartingResolution, TRUE);
  Grid.Enabled := FALSE;
  fNumberButtons.EnableButtons(FALSE);
  DroppingNumber := 0;

  //Rsolution
  if Sudoku.Start then
  begin
    //Cre la grille d'affichage
    SetLength(ResolvedGrid, 0);

    for Row := 0 to Grid.RowCount -1 do
      for Col := 0 to Grid.ColCount -1 do
        if Grid.Cells[Col, Row] = '' then
        begin
          SetLength(ResolvedGrid, Length(ResolvedGrid) +1);
          ResolvedGrid[High(ResolvedGrid)] := Point(Col, Row);
          Grid.Cells[Col, Row] := IntToStr(Sudoku.GetValue(Col, Row));
        end;

    //Mlange l'ordre d'affichage pour plus de ralisme
    if Length(ResolvedGrid) > 0 then
    begin
    for i := 0 to High(ResolvedGrid) do
      begin
        i1 := Random(high(ResolvedGrid));
        i2 := Random(high(ResolvedGrid));

        Pt := ResolvedGrid[i1];
        ResolvedGrid[i1] := ResolvedGrid[i2];
        ResolvedGrid[i2] := Pt;
      end;
    end;

    Play(SUDOKU_RESOLVED);
  end
  else Play(SUDOKU_FAIL);
end;

procedure TfMain.WMAfterShow(var Message: TMessage);
begin
  with Grid.ClientToScreen(Point(0,0)) do
  begin
    fDoor.Left := X -20;
    fDoor.Top  := Y -20;
    fDoor.Show;
  end;

  fHAL.Left := Left +420;
  fHAL.Top  := Top -50;
  fHAL.Show;

  fDisplay.Left := Left +420;
  fDisplay.Top  := Top +190;
  fDisplay.Show;

  fNumberButtons.Left := fDoor.Left -30;
  fNumberButtons.Top  := fDoor.BoundsRect.Bottom +20;
  fNumberButtons.Show;

  fCommandButtons.Left := fDisplay.Left;
  fCommandButtons.Top  := fDoor.BoundsRect.Bottom -fCommandButtons.Height;
  fCommandButtons.Show;

  fHAL.Play(HAL_ON);
  PlayRendering(DRAW_GRID);
  PlayRendering(DRAW_BUTTONS);
end;

procedure TfMain.bOffClick(Sender: TObject);
begin
  inherited;
  bPlay.Enabled := FALSE;
  bOFF.Enabled  := FALSE;

  Play(PLAY_QUIT);
end;

procedure TfMain.bOffMouseEnter(Sender: TObject);
begin
  inherited;

  if bOFF.Enabled then
    fHAL.Play(HAL_TALKING_JUSTWHAT);
end;

procedure TfMain.ApplicationEvents1Activate(Sender: TObject);
begin
  inherited;
  RestoreTopMostsEx;
end;

procedure TfMain.ApplicationEvents1Deactivate(Sender: TObject);
begin
  inherited;
  NormalizeTopMostsEx;
end;

procedure TfMain.FormCreate(Sender: TObject);
begin
  inherited;

  PlaySound(0, 'STARTUP');

  Sudoku := TSudoku.Create;

  BkBmp             := LoadPngToBmp('BACKGROUND');
  BtnBmp            := LoadPngToBmp('ROUND_BUTTON');
  PurpleBtnBmp      := LoadPngToBmp('ROUND_PURPLE');
  PurpleShadeBtnBmp := LoadPngToBmp('ROUND_PURPLE_SHADE');

  Background.SetBitmap(BkBmp);
end;

procedure TfMain.FormDestroy(Sender: TObject);
begin
  Sudoku.Free;
  BkBmp.Free;
  BtnBmp.Free;
  PurpleBtnBmp.Free;
  PurpleShadeBtnBmp.Free;

  inherited;
end;

procedure TfMain.FormShow(Sender: TObject);
begin
  inherited;
  PostMessage(Handle, WM_AFTERSHOW, 0, 0);
end;

initialization
  Randomize;

end.
