//Andnotor, developpez.net Defi N5 Sudoku solver
//------------------------------------------------------------------------------
//Fiche des boutons numriques (Voir aussi TfLayeredForm)
//------------------------------------------------------------------------------
unit NumberButtons;

interface

uses
  Effects, LayeredImage,
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, LayeredForm, Buttons,
  //SpeedButtonEx doit obligatoirement tre en dernier!
  SpeedButtonEx;

type
  TfNumberButtons = class(TfLayeredForm)
    bNb6: TSpeedButton;
    bNb8: TSpeedButton;
    bNb9: TSpeedButton;
    bNb5: TSpeedButton;
    bNb1: TSpeedButton;
    bNb3: TSpeedButton;
    bNb2: TSpeedButton;
    bNb4: TSpeedButton;
    bNb7: TSpeedButton;
    SpeedButton1: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure bNbClick(Sender: TObject);

  private
    BkBmp           :TBitmap;       //Image de fond
    BtnBmp          :TBitmap;       //Image des boutons
    BlueBtnBmp      :TBitmap;       //Couleur des boutons
    CoverBmp        :TBitmap;       //Porte opaque
    CoverLightBmp   :TBitmap;       //Porte transparente

  protected
    //Traitement des effets visuels et sonores.
    procedure DoEffect(const aEffect :TWMRendering); override;

  public
    ///(Ds)activation des boutons
    procedure EnableButtons(aEnabled :boolean);

  end;

var
  fNumberButtons: TfNumberButtons;

//------------------------------------------------------------------------------

implementation

uses EffectsConst, Main, HAL, Display;

{$R *.dfm}

procedure TfNumberButtons.bNbClick(Sender: TObject);
begin
  inherited;
  fMain.DroppingNumber := TControl(Sender).Tag;
end;

procedure TfNumberButtons.DoEffect(const aEffect: TWMRendering);
var
  //Permet d'inteprter aEffect comme un TWMSound sans transtypage
  Sound :TWMSound absolute aEffect;

  Bmp :TBitmap;

  R1, R2 :TRect;
  x, i   :integer;
begin
  inherited;

  if aEffect.State = esPlay then
  begin
    case aEffect.ID of
      PLAY_START     : //** Activation **
                       //- La porte opaque disparait
                       //- La porte transparente apparait
                       //  <synchro>
                       //- Son d'ouverture
                       //- La porte s'ouvre
                       //  <synchro>
                       //- Activation des boutons
                       PlayRendering(aEffect.ID, 30);

      PLAY_STOP      : //** Dsactivation **
                       //- Dsactivation des boutons
                       //  <synchro>
                       //- La porte se ferme
                       //  <synchro>
                       //- Son de fermeture
                       //- La porte opaque apparait
                       //- La porte transparente disparait
                       PlayRendering(aEffect.ID, ControlCount, 0, 100);

      DRAW_BUTTON    : //Dessin d'un bouton
                       PlayRendering(aEffect.ID, 1, aEffect.Param);
    end;

    Exit;
  end;

  case aEffect.ID of

    //-- Activation ------------------------------------------------------------

    PLAY_START1:        case aEffect.State of
                          esRenderingStart: begin
                                              fDisplay.Write(sActivateControls);
                                              Background.Pop;
                                            end;

                          esRenderingRun:   //La porte opaque disparait et la
                                            //transparente apparait
                                            begin
                                              Background.Draw(CoverBmp, erAll, StepDown(aEffect));
                                              Background.Draw(CoverLightBmp, erNone, StepUp(aEffect));
                                            end;

                          esRenderingEnd:   begin
                                              Sleep(500);
                                              PlayRendering(PLAY_START2, 50);
                                            end;
                        end;

    PLAY_START2:        case aEffect.State of
                          esRenderingStart: //Son
                                            PlaySound(0, 'SLIDEDOOR');

                          esRenderingRun:   //La porte coulisse
                                            begin
                                              x  := StepDown(aEffect, CoverBmp.Width);
                                              R1 := Rect(0, 0, CoverBmp.Width, CoverBmp.Height);
                                              R2 := Rect(0, 0, x, CoverBmp.Height);

                                              Background.Draw(CoverLightBmp, R1, R2, erAll);
                                            end;

                          esRenderingEnd:   PlayRendering(PLAY_START3, ControlCount, 0, 100);
                        end;

    PLAY_START3:        case aEffect.State of
                          esRenderingRun:   //Activation des boutons
                                            Controls[aEffect.Step -1].Enabled := TRUE;

                          esRenderingEnd:   begin
                                              fDisplay.WriteLn(sSuccess);
                                              fDisplay.WriteLn(sReadyToPlay, TRUE);
                                            end;
                        end;

    //-- Dactivation ----------------------------------------------------------

    PLAY_STOP1:         case aEffect.State of
                          esRenderingStart: fDisplay.Write(sDesactivateControls);

                          esRenderingRun:   //Dsactive les boutons
                                            Controls[aEffect.Step -1].Enabled := FALSE;

                          esRenderingEnd:   begin
                                              Background.Push;
                                              Sleep(1000);
                                              PlayRendering(PLAY_STOP2, 50);
                                            end;
                        end;

    PLAY_STOP2:         case aEffect.State of
                          esRenderingStart: //Son
                                            PlaySound(0, 'SLIDEDOOR');

                          esRenderingRun:   //La porte se ferme
                                            begin
                                              x  := StepUp(aEffect, CoverBmp.Width);
                                              R1 := Rect(0, 0, CoverBmp.Width, CoverBmp.Height);
                                              R2 := Rect(0, 0, x, CoverBmp.Height);
                                              Background.Draw(CoverLightBmp, R1, R2, erDest);
                                            end;

                          esRenderingEnd:   begin
                                              Sleep(500);
                                              PlayRendering(PLAY_STOP3, 30);
                                            end;
                        end;

    PLAY_STOP3:         case aEffect.State of
                          esRenderingRun:   //La porte opaque apparait et la
                                            //transparente disparait
                                            begin
                                              Background.Draw(CoverBmp, erAll, StepUp(aEffect));
                                              Background.Draw(CoverLightBmp, erNone, StepDown(aEffect));
                                            end;

                          esRenderingEnd:   begin
                                              fDisplay.WriteLn(sSuccess);
                                              fMain.Play(PLAY_STOP2);
                                            end;
                        end;

    //-- Dessin des boutons dsactivs -----------------------------------------

    DRAW_BUTTONS:       case aEffect.State of
                          esRenderingRun:   for i := 0 to ControlCount -1 do
                                            begin
                                              R1 := Rect(0, 0, BtnBmp.Width, BtnBmp.Height);
                                              R2 := Controls[i].BoundsRect;

                                              Background.Draw(BtnBmp, R1, R2);
                                            end;

                          esRenderingEnd:   begin
                                              Background.Push;
                                              Background.Draw(CoverBmp);
                                              Background.Push;
                                            end;

                        end;

    //-- Dessine un bouton (ds)activs ----------------------------------------

    DRAW_BUTTON:        case aEffect.State of
                          esRenderingRun:   with TSpeedButton(aEffect.Param) do
                                            begin
                                              if Enabled then
                                              begin
                                                Bmp := LoadPngToBmp(Format('NB%d_WHITE', [Tag]));

                                                R1 := Rect(0, 0, BtnBmp.Width, BtnBmp.Height);

                                                if Down then
                                                begin
                                                  Background.Draw(BlueBtnBmp, R1, BoundsRect, erDest);
                                                  Background.Draw(Bmp, R1, BoundsRect, erNone, 200);
                                                end
                                                else
                                                begin
                                                  Background.Draw(BlueBtnBmp, R1, BoundsRect, erDest, 150);
                                                  Background.Draw(Bmp, R1, BoundsRect, erNone, 100);
                                                end;

                                                Bmp.Free;
                                              end
                                              else Background.Erase(BoundsRect);
                                            end;
                        end;
  end;

  //Force la nouvelle image  l'cran
  Background.Flush;
end;

procedure TfNumberButtons.EnableButtons(aEnabled: boolean);
var
  i: Integer;
begin
  for i := 0 to ControlCount -1 do
    with TSpeedButton(Controls[i]) do
    begin
      Down    := FALSE;
      Enabled := aEnabled;
    end;
end;

procedure TfNumberButtons.FormCreate(Sender: TObject);
begin
  inherited;

  BkBmp           := LoadPngToBmp('NUMBER_BTN_BK');
  BtnBmp          := LoadPngToBmp('ROUND_BUTTON');
  BlueBtnBmp      := LoadPngToBmp('ROUND_BLUE');
  CoverBmp        := LoadPngToBmp('NUMBER_COVER');
  CoverLightBmp   := LoadPngToBmp('NUMBER_TRANS_COVER');

  Background.SetBitmap(BkBmp);
  Background.Flush;
  PlayRendering(DRAW_BUTTONS);
end;

procedure TfNumberButtons.FormDestroy(Sender: TObject);
begin
  BkBmp.Free;
  BtnBmp.Free;
  BlueBtnBmp.Free;
  CoverBmp.Free;
  CoverLightBmp.Free;

  inherited;
end;

end.
